use std::io;
use std::path::{Path, PathBuf};

use anyhow::Result;
use tracing::trace;

use super::v1;


#[derive(Debug, Clone, PartialEq, serde::Serialize, serde::Deserialize)]
pub struct Project {
    pub path: PathBuf,
    pub custom_name: Option<String>,
}

impl Project {
    pub fn path(&self) -> &PathBuf {
        &self.path
    }

    pub fn name(&self) -> String {
        match &self.custom_name {
            Some(name) => name.clone(),
            None => {
                let name = self.path.file_name().unwrap();
                name.to_string_lossy().to_string()
            }
        }
    }
}

impl From<PathBuf> for Project {
    fn from(path: PathBuf) -> Self {
        Self {
            path,
            custom_name: None,
        }
    }
}

impl From<v1::Project> for Project {
    fn from(value: v1::Project) -> Self {
        Self {
            path: value.path,
            custom_name: None,
        }
    }
}


#[tracing::instrument(skip_all)]
pub fn deserialize_project_list<P>(path: P) -> Result<Vec<super::Project>>
where
    P: AsRef<Path>
{
    let file = std::fs::File::open(path)?;

    trace!("Deserializing V2 project list");
    let list = bincode::deserialize_from::<_, Vec<super::Project>>(file)?;

    Ok(list)
}
